@@ -1,18 +1,20 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille and Sylvain Corlay                     *
+* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XARRAY_HPP
-#define XARRAY_HPP
+#ifndef XTENSOR_ARRAY_HPP
+#define XTENSOR_ARRAY_HPP
 
 #include <algorithm>
 #include <initializer_list>
 #include <utility>
 
+#include "xtl/xsequence.hpp"
+
 #include "xcontainer.hpp"
 #include "xsemantic.hpp"
 
@@ -23,8 +25,8 @@ namespace xt
      * xarray_container declaration *
      ********************************/
 
-    template <class EC, layout_type L, class SC>
-    struct xcontainer_inner_types<xarray_container<EC, L, SC>>
+    template <class EC, layout_type L, class SC, class Tag>
+    struct xcontainer_inner_types<xarray_container<EC, L, SC, Tag>>
     {
         using container_type = EC;
         using shape_type = SC;
@@ -33,12 +35,13 @@ namespace xt
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xarray_container<EC, L, SC>;
+        using temporary_type = xarray_container<EC, L, SC, Tag>;
+        static constexpr layout_type layout = L;
     };
 
-    template <class EC, layout_type L, class SC>
-    struct xiterable_inner_types<xarray_container<EC, L, SC>>
-        : xcontainer_iterable_types<xarray_container<EC, L, SC>>
+    template <class EC, layout_type L, class SC, class Tag>
+    struct xiterable_inner_types<xarray_container<EC, L, SC, Tag>>
+        : xcontainer_iterable_types<xarray_container<EC, L, SC, Tag>>
     {
     };
 
@@ -52,19 +55,20 @@ namespace xt
      * @tparam EC The type of the container holding the elements.
      * @tparam L The layout_type of the container.
      * @tparam SC The type of the containers holding the shape and the strides.
+     * @tparam Tag The expression tag.
      * @sa xarray
      */
-    template <class EC, layout_type L, class SC>
-    class xarray_container : public xstrided_container<xarray_container<EC, L, SC>, L>,
-                             public xcontainer_semantic<xarray_container<EC, L, SC>>
+    template <class EC, layout_type L, class SC, class Tag>
+    class xarray_container : public xstrided_container<xarray_container<EC, L, SC, Tag>>,
+                             public xcontainer_semantic<xarray_container<EC, L, SC, Tag>>
     {
-
     public:
 
-        using self_type = xarray_container<EC, L, SC>;
-        using base_type = xstrided_container<self_type, L>;
+        using self_type = xarray_container<EC, L, SC, Tag>;
+        using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
         using container_type = typename base_type::container_type;
+        using allocator_type = typename base_type::allocator_type;
         using value_type = typename base_type::value_type;
         using reference = typename base_type::reference;
         using const_reference = typename base_type::const_reference;
@@ -73,7 +77,10 @@ namespace xt
         using shape_type = typename base_type::shape_type;
         using inner_shape_type = typename base_type::inner_shape_type;
         using strides_type = typename base_type::strides_type;
+        using backstrides_type = typename base_type::backstrides_type;
         using inner_strides_type = typename base_type::inner_strides_type;
+        using temporary_type = typename semantic_base::temporary_type;
+        using expression_tag = Tag;
 
         xarray_container();
         explicit xarray_container(const shape_type& shape, layout_type l = L);
@@ -89,6 +96,9 @@ namespace xt
         xarray_container(nested_initializer_list_t<value_type, 4> t);
         xarray_container(nested_initializer_list_t<value_type, 5> t);
 
+        template <class S = shape_type>
+        static xarray_container from_shape(S&& s);
+
         ~xarray_container() = default;
 
         xarray_container(const xarray_container&) = default;
@@ -110,32 +120,30 @@ namespace xt
         container_type& data_impl() noexcept;
         const container_type& data_impl() const noexcept;
 
-        friend class xcontainer<xarray_container<EC, L, SC>>;
+        friend class xcontainer<xarray_container<EC, L, SC, Tag>>;
     };
 
     /******************************
      * xarray_adaptor declaration *
      ******************************/
 
-    template <class EC, layout_type L = DEFAULT_LAYOUT, class SC = std::vector<typename EC::size_type>>
-    class xarray_adaptor;
-
-    template <class EC, layout_type L, class SC>
-    struct xcontainer_inner_types<xarray_adaptor<EC, L, SC>>
+    template <class EC, layout_type L, class SC, class Tag>
+    struct xcontainer_inner_types<xarray_adaptor<EC, L, SC, Tag>>
     {
-        using container_type = EC;
+        using container_type = std::remove_reference_t<EC>;
         using shape_type = SC;
         using strides_type = shape_type;
         using backstrides_type = shape_type;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xarray_container<EC, L, SC>;
+        using temporary_type = xarray_container<container_type, L, SC, Tag>;
+        static constexpr layout_type layout = L;
     };
 
-    template <class EC, layout_type L, class SC>
-    struct xiterable_inner_types<xarray_adaptor<EC, L, SC>>
-        : xcontainer_iterable_types<xarray_adaptor<EC, L, SC>>
+    template <class EC, layout_type L, class SC, class Tag>
+    struct xiterable_inner_types<xarray_adaptor<EC, L, SC, Tag>>
+        : xcontainer_iterable_types<xarray_adaptor<EC, L, SC, Tag>>
     {
     };
 
@@ -149,27 +157,37 @@ namespace xt
      * a multidimensional container semantic and a tensor semantic to
      * stl-like containers.
      *
-     * @tparam EC The container type to adapt.
+     * @tparam EC The closure for the container type to adapt.
      * @tparam L The layout_type of the adaptor.
      * @tparam SC The type of the containers holding the shape and the strides.
+     * @tparam Tag The expression tag.
      */
-    template <class EC, layout_type L, class SC>
-    class xarray_adaptor : public xstrided_container<xarray_adaptor<EC, L, SC>, L>,
-                           public xadaptor_semantic<xarray_adaptor<EC, L, SC>>
+    template <class EC, layout_type L, class SC, class Tag>
+    class xarray_adaptor : public xstrided_container<xarray_adaptor<EC, L, SC, Tag>>,
+                           public xcontainer_semantic<xarray_adaptor<EC, L, SC, Tag>>
     {
-
     public:
 
-        using self_type = xarray_adaptor<EC, L, SC>;
-        using base_type = xstrided_container<self_type, L>;
-        using semantic_base = xadaptor_semantic<self_type>;
+        using container_closure_type = EC;
+
+        using self_type = xarray_adaptor<EC, L, SC, Tag>;
+        using base_type = xstrided_container<self_type>;
+        using semantic_base = xcontainer_semantic<self_type>;
         using container_type = typename base_type::container_type;
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
+        using backstrides_type = typename base_type::backstrides_type;
+        using temporary_type = typename semantic_base::temporary_type;
+        using expression_tag = Tag;
 
-        xarray_adaptor(container_type& data);
-        xarray_adaptor(container_type& data, const shape_type& shape, layout_type l = L);
-        xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides);
+        xarray_adaptor(container_type&& data);
+        xarray_adaptor(const container_type& data);
+
+        template <class D>
+        xarray_adaptor(D&& data, const shape_type& shape, layout_type l = L);
+
+        template <class D>
+        xarray_adaptor(D&& data, const shape_type& shape, const strides_type& strides);
 
         ~xarray_adaptor() = default;
 
@@ -178,23 +196,20 @@ namespace xt
 
         xarray_adaptor(xarray_adaptor&&) = default;
         xarray_adaptor& operator=(xarray_adaptor&&);
+        xarray_adaptor& operator=(temporary_type&&);
 
         template <class E>
         xarray_adaptor& operator=(const xexpression<E>& e);
 
     private:
 
-        container_type& m_data;
+        container_closure_type m_data;
 
         container_type& data_impl() noexcept;
         const container_type& data_impl() const noexcept;
 
-        using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
-        void assign_temporary_impl(temporary_type& tmp);
 
-
-        friend class xcontainer<xarray_adaptor<EC, L, SC>>;
-        friend class xadaptor_semantic<xarray_adaptor<EC, L, SC>>;
+        friend class xcontainer<xarray_adaptor<EC, L, SC, Tag>>;
     };
 
     /***********************************
@@ -208,8 +223,8 @@ namespace xt
     /**
      * Allocates an uninitialized xarray_container that holds 0 element.
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container()
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container()
         : base_type(), m_data(1, value_type())
     {
     }
@@ -220,11 +235,11 @@ namespace xt
      * @param shape the shape of the xarray_container
      * @param l the layout_type of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(const shape_type& shape, layout_type l)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, layout_type l)
         : base_type()
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
     }
 
     /**
@@ -234,11 +249,11 @@ namespace xt
      * @param value the value of the elements
      * @param l the layout_type of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(const shape_type& shape, const_reference value, layout_type l)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, const_reference value, layout_type l)
         : base_type()
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
         std::fill(m_data.begin(), m_data.end(), value);
     }
 
@@ -247,11 +262,11 @@ namespace xt
      * @param shape the shape of the xarray_container
      * @param strides the strides of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(const shape_type& shape, const strides_type& strides)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, const strides_type& strides)
         : base_type()
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
     }
 
     /**
@@ -261,11 +276,11 @@ namespace xt
      * @param strides the strides of the xarray_container
      * @param value the value of the elements
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(const shape_type& shape, const strides_type& strides, const_reference value)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(const shape_type& shape, const strides_type& strides, const_reference value)
         : base_type()
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
         std::fill(m_data.begin(), m_data.end(), value);
     }
 
@@ -274,11 +289,11 @@ namespace xt
      * specified value.
      * @param t the value of the element
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(const value_type& t)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(const value_type& t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t), true);
+        base_type::resize(xt::shape<shape_type>(t), true);
         nested_copy(m_data.begin(), t);
     }
 
@@ -289,8 +304,8 @@ namespace xt
      * @param shape the shape of the xarray_container
      * @param strides the strides of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides)
         : base_type(std::move(shape), std::move(strides)), m_data(std::move(data))
     {
     }
@@ -304,63 +319,74 @@ namespace xt
      * Allocates a one-dimensional xarray_container.
      * @param t the elements of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(nested_initializer_list_t<value_type, 1> t)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 1> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->xbegin(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
      * Allocates a two-dimensional xarray_container.
      * @param t the elements of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(nested_initializer_list_t<value_type, 2> t)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 2> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->xbegin(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
      * Allocates a three-dimensional xarray_container.
      * @param t the elements of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(nested_initializer_list_t<value_type, 3> t)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 3> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->xbegin(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
      * Allocates a four-dimensional xarray_container.
      * @param t the elements of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(nested_initializer_list_t<value_type, 4> t)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 4> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->xbegin(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
 
     /**
      * Allocates a five-dimensional xarray_container.
      * @param t the elements of the xarray_container
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_container<EC, L, SC>::xarray_container(nested_initializer_list_t<value_type, 5> t)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(nested_initializer_list_t<value_type, 5> t)
         : base_type()
     {
-        base_type::reshape(xt::shape<shape_type>(t));
-        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->xbegin(), t);
+        base_type::resize(xt::shape<shape_type>(t));
+        L == layout_type::row_major ? nested_copy(m_data.begin(), t) : nested_copy(this->template begin<layout_type::row_major>(), t);
     }
     //@}
 
+    /**
+     * Allocates and returns an xarray_container with the specified shape.
+     * @param s the shape of the xarray_container
+     */
+    template <class EC, layout_type L, class SC, class Tag>
+    template <class S>
+    inline xarray_container<EC, L, SC, Tag> xarray_container<EC, L, SC, Tag>::from_shape(S&& s)
+    {
+        shape_type shape = xtl::forward_sequence<shape_type>(s);
+        return self_type(shape);
+    }
 
     /**
      * @name Extended copy semantic
@@ -369,13 +395,13 @@ namespace xt
     /**
      * The extended copy constructor.
      */
-    template <class EC, layout_type L, class SC>
+    template <class EC, layout_type L, class SC, class Tag>
     template <class E>
-    inline xarray_container<EC, L, SC>::xarray_container(const xexpression<E>& e)
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(const xexpression<E>& e)
         : base_type()
     {
         // Avoids unintialized data because of (m_shape == shape) condition
-        // in reshape (called by assign), which is always true when dimension == 0.
+        // in resize (called by assign), which is always true when dimension == 0.
         if (e.derived_cast().dimension() == 0)
         {
             m_data.resize(1);
@@ -386,22 +412,22 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class EC, layout_type L, class SC>
+    template <class EC, layout_type L, class SC, class Tag>
     template <class E>
-    inline auto xarray_container<EC, L, SC>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xarray_container<EC, L, SC, Tag>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class EC, layout_type L, class SC>
-    inline auto xarray_container<EC, L, SC>::data_impl() noexcept -> container_type&
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_container<EC, L, SC, Tag>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class EC, layout_type L, class SC>
-    inline auto xarray_container<EC, L, SC>::data_impl() const noexcept -> const container_type&
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_container<EC, L, SC, Tag>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -418,8 +444,18 @@ namespace xt
      * Constructs an xarray_adaptor of the given stl-like container.
      * @param data the container to adapt
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_adaptor<EC, L, SC>::xarray_adaptor(container_type& data)
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(container_type&& data)
+        : base_type(), m_data(std::move(data))
+    {
+    }
+
+    /**
+     * Constructs an xarray_adaptor of the given stl-like container.
+     * @param data the container to adapt
+     */
+    template <class EC, layout_type L, class SC, class Tag>
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(const container_type& data)
         : base_type(), m_data(data)
     {
     }
@@ -431,11 +467,12 @@ namespace xt
      * @param shape the shape of the xarray_adaptor
      * @param l the layout_type of the xarray_adaptor
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_adaptor<EC, L, SC>::xarray_adaptor(container_type& data, const shape_type& shape, layout_type l)
-        : base_type(), m_data(data)
+    template <class EC, layout_type L, class SC, class Tag>
+    template <class D>
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(D&& data, const shape_type& shape, layout_type l)
+        : base_type(), m_data(std::forward<D>(data))
     {
-        base_type::reshape(shape, l);
+        base_type::resize(shape, l);
     }
 
     /**
@@ -445,30 +482,41 @@ namespace xt
      * @param shape the shape of the xarray_adaptor
      * @param strides the strides of the xarray_adaptor
      */
-    template <class EC, layout_type L, class SC>
-    inline xarray_adaptor<EC, L, SC>::xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
-        : base_type(), m_data(data)
+    template <class EC, layout_type L, class SC, class Tag>
+    template <class D>
+    inline xarray_adaptor<EC, L, SC, Tag>::xarray_adaptor(D&& data, const shape_type& shape, const strides_type& strides)
+        : base_type(), m_data(std::forward<D>(data))
     {
-        base_type::reshape(shape, strides);
+        base_type::resize(shape, strides);
     }
     //@}
 
-    template <class EC, layout_type L, class SC>
-    inline auto xarray_adaptor<EC, L, SC>::operator=(const xarray_adaptor& rhs) -> self_type&
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_adaptor<EC, L, SC, Tag>::operator=(const xarray_adaptor& rhs) -> self_type&
     {
         base_type::operator=(rhs);
         m_data = rhs.m_data;
         return *this;
     }
 
-    template <class EC, layout_type L, class SC>
-    inline auto xarray_adaptor<EC, L, SC>::operator=(xarray_adaptor&& rhs) -> self_type&
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_adaptor<EC, L, SC, Tag>::operator=(xarray_adaptor&& rhs) -> self_type&
     {
         base_type::operator=(std::move(rhs));
         m_data = rhs.m_data;
         return *this;
     }
 
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_adaptor<EC, L, SC, Tag>::operator=(temporary_type&& rhs) -> self_type&
+    {
+        base_type::shape_impl() = std::move(const_cast<shape_type&>(rhs.shape()));
+        base_type::strides_impl() = std::move(const_cast<strides_type&>(rhs.strides()));
+        base_type::backstrides_impl() = std::move(const_cast<backstrides_type&>(rhs.backstrides()));
+        m_data = std::move(rhs.data());
+        return *this;
+    }
+
     /**
      * @name Extended copy semantic
      */
@@ -476,37 +524,25 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class EC, layout_type L, class SC>
+    template <class EC, layout_type L, class SC, class Tag>
     template <class E>
-    inline auto xarray_adaptor<EC, L, SC>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xarray_adaptor<EC, L, SC, Tag>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class EC, layout_type L, class SC>
-    inline auto xarray_adaptor<EC, L, SC>::data_impl() noexcept -> container_type&
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_adaptor<EC, L, SC, Tag>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class EC, layout_type L, class SC>
-    inline auto xarray_adaptor<EC, L, SC>::data_impl() const noexcept -> const container_type&
+    template <class EC, layout_type L, class SC, class Tag>
+    inline auto xarray_adaptor<EC, L, SC, Tag>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
-
-    template <class EC, layout_type L, class SC>
-    inline void xarray_adaptor<EC, L, SC>::assign_temporary_impl(temporary_type& tmp)
-    {
-        // TODO (performance improvement) : consider moving tmps
-        // shape and strides
-        base_type::shape_impl() = tmp.shape();
-        base_type::strides_impl() = tmp.strides();
-        base_type::backstrides_impl() = tmp.backstrides();
-        m_data.resize(tmp.size());
-        std::copy(tmp.data().cbegin(), tmp.data().cend(), m_data.begin());
-    }
 }
 
 #endif
