@@ -79,7 +79,7 @@ namespace xt
     private:
 
         template <class E1, class E2>
-        static bool reshape(xexpression<E1>& e1, const xexpression<E2>& e2);
+        static bool resize(xexpression<E1>& e1, const xexpression<E2>& e2);
     };
 
     /*****************
@@ -96,12 +96,13 @@ namespace xt
         using shape_type = typename E1::shape_type;
         using index_type = xindex_type_t<shape_type>;
         using size_type = typename lhs_iterator::size_type;
+        using difference_type = typename lhs_iterator::difference_type;
 
         data_assigner(E1& e1, const E2& e2);
 
         void run();
 
-        void step(size_type i);
+        void step(size_type i, size_type n);
         void reset(size_type i);
 
         void to_end(layout_type);
@@ -226,7 +227,7 @@ namespace xt
     template <class E1, class E2>
     inline void xexpression_assigner<Tag>::assign_xexpression(xexpression<E1>& e1, const xexpression<E2>& e2)
     {
-        bool trivial_broadcast = reshape(e1, e2);
+        bool trivial_broadcast = resize(e1, e2);
         base_type::assign_data(e1, e2, trivial_broadcast);
     }
 
@@ -284,7 +285,7 @@ namespace xt
 
     template <class Tag>
     template <class E1, class E2>
-    inline bool xexpression_assigner<Tag>::reshape(xexpression<E1>& e1, const xexpression<E2>& e2)
+    inline bool xexpression_assigner<Tag>::resize(xexpression<E1>& e1, const xexpression<E2>& e2)
     {
         using shape_type = typename E1::shape_type;
         using size_type = typename E1::size_type;
@@ -292,7 +293,7 @@ namespace xt
         size_type size = de2.dimension();
         shape_type shape = xtl::make_sequence<shape_type>(size, size_type(1));
         bool trivial_broadcast = de2.broadcast_shape(shape);
-        e1.derived_cast().reshape(std::move(shape));
+        e1.derived_cast().resize(std::move(shape));
         return trivial_broadcast;
     }
 
@@ -323,10 +324,10 @@ namespace xt
     }
 
     template <class E1, class E2, layout_type L>
-    inline void data_assigner<E1, E2, L>::step(size_type i)
+    inline void data_assigner<E1, E2, L>::step(size_type i, size_type n)
     {
-        m_lhs.step(i);
-        m_rhs.step(i);
+        m_lhs.step(i, n);
+        m_rhs.step(i, n);
     }
 
     template <class E1, class E2, layout_type L>
@@ -359,8 +360,10 @@ namespace xt
         using size_type = typename E1::size_type;
         size_type size = e1.size();
         size_type simd_size = simd_type::size;
+
         size_type align_begin = is_aligned ? 0 : xsimd::get_alignment_offset(e1.raw_data(), size, simd_size);
         size_type align_end = align_begin + ((size - align_begin) & ~(simd_size - 1));
+
         for (size_type i = 0; i < align_begin; ++i)
         {
             e1.data_element(i) = e2.data_element(i);
@@ -380,7 +383,7 @@ namespace xt
         template <class E1, class E2>
         inline void trivial_assigner_run_impl(E1& e1, const E2& e2, std::true_type)
         {
-            std::copy(e2.storage_cbegin(), e2.storage_cend(), e1.storage_begin());
+            std::transform(e2.storage_cbegin(), e2.storage_cend(), e1.storage_begin(), [](typename E2::value_type x) { return static_cast<typename E1::value_type>(x); });
         }
 
         template <class E1, class E2>
