@@ -58,6 +58,7 @@ namespace xt
 
         using inner_types = xcontainer_inner_types<D>;
         using container_type = typename inner_types::container_type;
+        using allocator_type = typename container_type::allocator_type;
         using value_type = typename container_type::value_type;
         using reference = typename container_type::reference;
         using const_reference = typename container_type::const_reference;
@@ -81,9 +82,11 @@ namespace xt
 
         static constexpr layout_type static_layout = inner_types::layout;
         static constexpr bool contiguous_layout = static_layout != layout_type::dynamic;
-        using data_alignment = xsimd::container_alignment<container_type>;
+        using data_alignment = xsimd::container_alignment_t<container_type>;
         using simd_type = xsimd::simd_type<value_type>;
 
+        static_assert(static_layout != layout_type::any, "Container layout can never be layout_type::any!");
+
         size_type size() const noexcept;
 
         constexpr size_type dimension() const noexcept;
@@ -361,11 +364,14 @@ namespace xt
         using inner_backstrides_type = typename base_type::inner_backstrides_type;
 
         template <class S = shape_type>
-        void reshape(S&& shape, bool force = false);
+        void resize(S&& shape, bool force = false);
         template <class S = shape_type>
-        void reshape(S&& shape, layout_type l);
+        void resize(S&& shape, layout_type l);
         template <class S = shape_type>
-        void reshape(S&& shape, const strides_type& strides);
+        void resize(S&& shape, const strides_type& strides);
+
+        template <class S = shape_type>
+        void reshape(S&& shape, layout_type layout = base_type::static_layout);
 
         layout_type layout() const noexcept;
 
@@ -504,7 +510,7 @@ namespace xt
     template <class... Args>
     inline auto xcontainer<D>::operator()(Args... args) -> reference
     {
-        XTENSOR_ASSERT(check_index(shape(), args...));
+        XTENSOR_TRY(check_index(shape(), args...));
         XTENSOR_CHECK_DIMENSION(shape(), args...);
         size_type index = data_offset<size_type>(strides(), static_cast<size_type>(args)...);
         return data()[index];
@@ -520,7 +526,7 @@ namespace xt
     template <class... Args>
     inline auto xcontainer<D>::operator()(Args... args) const -> const_reference
     {
-        XTENSOR_ASSERT(check_index(shape(), args...));
+        XTENSOR_TRY(check_index(shape(), args...));
         XTENSOR_CHECK_DIMENSION(shape(), args...);
         size_type index = data_offset<size_type>(strides(), static_cast<size_type>(args)...);
         return data()[index];
@@ -625,7 +631,7 @@ namespace xt
     template <class It>
     inline auto xcontainer<D>::element(It first, It last) -> reference
     {
-        XTENSOR_ASSERT(check_element_index(shape(), first, last));
+        XTENSOR_TRY(check_element_index(shape(), first, last));
         return data()[element_offset<size_type>(strides(), first, last)];
     }
 
@@ -640,7 +646,7 @@ namespace xt
     template <class It>
     inline auto xcontainer<D>::element(It first, It last) const -> const_reference
     {
-        XTENSOR_ASSERT(check_element_index(shape(), first, last));
+        XTENSOR_TRY(check_element_index(shape(), first, last));
         return data()[element_offset<size_type>(strides(), first, last)];
     }
 
@@ -1198,13 +1204,13 @@ namespace xt
     }
 
     /**
-     * Reshapes the container.
+     * resizes the container.
      * @param shape the new shape
      * @param force force reshaping, even if the shape stays the same (default: false)
      */
     template <class D>
     template <class S>
-    inline void xstrided_container<D>::reshape(S&& shape, bool force)
+    inline void xstrided_container<D>::resize(S&& shape, bool force)
     {
         if (m_shape.size() != shape.size() || !std::equal(std::begin(shape), std::end(shape), std::begin(m_shape)) || force)
         {
@@ -1221,34 +1227,34 @@ namespace xt
     }
 
     /**
-     * Reshapes the container.
+     * resizes the container.
      * @param shape the new shape
      * @param l the new layout_type
      */
     template <class D>
     template <class S>
-    inline void xstrided_container<D>::reshape(S&& shape, layout_type l)
+    inline void xstrided_container<D>::resize(S&& shape, layout_type l)
     {
         if (base_type::static_layout != layout_type::dynamic && l != base_type::static_layout)
         {
             throw std::runtime_error("Cannot change layout_type if template parameter not layout_type::dynamic.");
         }
         m_layout = l;
-        reshape(std::forward<S>(shape), true);
+        resize(std::forward<S>(shape), true);
     }
 
     /**
-     * Reshapes the container.
+     * Resizes the container.
      * @param shape the new shape
      * @param strides the new strides
      */
     template <class D>
     template <class S>
-    inline void xstrided_container<D>::reshape(S&& shape, const strides_type& strides)
+    inline void xstrided_container<D>::resize(S&& shape, const strides_type& strides)
     {
         if (base_type::static_layout != layout_type::dynamic)
         {
-            throw std::runtime_error("Cannot reshape with custom strides when layout() is != layout_type::dynamic.");
+            throw std::runtime_error("Cannot resize with custom strides when layout() is != layout_type::dynamic.");
         }
         m_shape = xtl::forward_sequence<shape_type>(shape);
         m_strides = strides;
@@ -1257,6 +1263,35 @@ namespace xt
         m_layout = layout_type::dynamic;
         detail::resize_data_container(this->data(), compute_size(m_shape));
     }
+
+    /**
+     * Reshapes the container and keeps old elements
+     * @param shape the new shape (has to have same number of elements as the original container)
+     * @param layout the layout to compute the strides (defaults to static layout of the container,
+     *               or for a container with dynamic layout to DEFAULT_LAYOUT)
+     */
+    template <class D>
+    template <class S>
+    inline void xstrided_container<D>::reshape(S&& shape, layout_type layout)
+    {
+        if (compute_size(shape) != this->size())
+        {
+            throw std::runtime_error("Cannot reshape with incorrect number of elements.");
+        }
+        if (layout == layout_type::dynamic || layout == layout_type::any)
+        {
+            layout = DEFAULT_LAYOUT;  // fall back to default layout
+        }
+        if (layout != base_type::static_layout && base_type::static_layout != layout_type::dynamic)
+        {
+            throw std::runtime_error("Cannot reshape with different layout if static layout != dynamic.");
+        }
+        m_layout = layout;
+        m_shape = xtl::forward_sequence<shape_type>(shape);
+        resize_container(m_strides, m_shape.size());
+        resize_container(m_backstrides, m_shape.size());
+        size_type data_size = compute_strides(m_shape, m_layout, m_strides, m_backstrides);
+    }
 }
 
 #endif
