@@ -37,20 +37,236 @@ namespace xt
      * reduce *
      **********/
 
-    template <class F, class E, class X>
-    auto reduce(F&& f, E&& e, X&& axes) noexcept;
+#define DEFAULT_STRATEGY_REDUCERS evaluation_strategy::lazy
 
-    template <class F, class E>
-    auto reduce(F&& f, E&& e) noexcept;
+    template <class F, class E, class X, class ES = DEFAULT_STRATEGY_REDUCERS,
+              class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value, int>>
+    auto reduce(F&& f, E&& e, X&& axes, ES es = ES()) noexcept;
+
+    template <class F, class E, class ES = DEFAULT_STRATEGY_REDUCERS,
+              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, ES>::value, int>>
+    auto reduce(F&& f, E&& e, ES es = ES()) noexcept;
 
 #ifdef X_OLD_CLANG
     template <class F, class E, class I>
     auto reduce(F&& f, E&& e, std::initializer_list<I> axes) noexcept;
 #else
-    template <class F, class E, class I, std::size_t N>
-    auto reduce(F&& f, E&& e, const I (&axes)[N]) noexcept;
+    template <class F, class E, class I, std::size_t N, class ES = DEFAULT_STRATEGY_REDUCERS>
+    auto reduce(F&& f, E&& e, const I (&axes)[N], ES es = ES()) noexcept;
 #endif
 
+    template <class F, class E, class X>
+    auto reduce_immediate(F&& f, E&& e, X&& axes)
+    {
+        using shape_type = std::vector<std::size_t>;
+        using accumulate_functor = std::decay_t<decltype(std::get<0>(f))>;
+        using result_type = typename accumulate_functor::result_type;
+
+        // retrieve functors from triple struct
+        auto acc_fct = std::get<0>(f);
+        auto init_fct = std::get<1>(f);
+        auto merge_fct = std::get<2>(f);
+
+        shape_type result_shape(e.dimension() - axes.size());
+        std::vector<std::size_t> iter_shape = e.shape();
+        std::vector<std::size_t> iter_strides(e.dimension());
+
+        xt::xarray<result_type, std::decay_t<E>::static_layout> result;
+
+        if (!std::is_sorted(axes.cbegin(), axes.cend()))
+        {
+            throw std::runtime_error("Reducing axes should be sorted");
+        }
+
+        // Fast track for complete reduction
+        if (e.dimension() == axes.size())
+        {
+            auto begin = e.data().begin();
+            result_type tmp = init_fct(*begin);
+            ++begin;
+            result(0) = std::accumulate(begin, e.data().end(), tmp, acc_fct);
+            return result;
+        }
+
+        // axis wise reductions:
+        for (std::size_t i = 0, idx = 0; i < e.dimension(); ++i)
+        {
+            if (std::find(axes.begin(), axes.end(), i) == axes.end())
+            {
+                // i not in axes!
+                result_shape[idx] = e.shape()[i];
+                ++idx;
+            }
+        }
+
+        result.resize(result_shape, e.layout());
+
+        std::size_t ax_idx = (e.layout() == layout_type::row_major) ? axes.size() - 1 : 0;
+        std::size_t inner_loop_size = e.strides()[axes[ax_idx]];
+        std::size_t inner_stride    = e.strides()[axes[ax_idx]];
+        std::size_t outer_loop_size = e.shape()[axes[ax_idx]];
+
+        // The following code merges reduction axes "at the end" (or the beginning for col_major)
+        // together by increasing the size of the outer loop where appropriate
+        auto merge_loops = [&outer_loop_size, &e](auto it, auto end)
+        {
+            auto last_ax = *it;
+            ++it;
+            for (; it != end; ++it)
+            {
+                // note that we check is_sorted, so this condition is valid
+                if (std::abs(ptrdiff_t(*it) - ptrdiff_t(last_ax)) ==  1)
+                {
+                    last_ax = *it;
+                    outer_loop_size *= e.shape()[last_ax];
+                }
+            }
+            return last_ax;
+        };
+
+        for (std::size_t i = 0, idx = 0; i < e.dimension(); ++i)
+        {
+            if (std::find(axes.begin(), axes.end(), i) == axes.end())
+            {
+                // i not in axes!
+                iter_strides[i] = result.strides()[idx];
+                ++idx;
+            }
+        }
+
+        if (e.layout() == layout_type::row_major)
+        {
+            std::size_t last_ax = merge_loops(axes.rbegin(), axes.rend());
+
+            iter_shape.erase(iter_shape.begin() + ptrdiff_t(last_ax), iter_shape.end());
+            iter_strides.erase(iter_strides.begin() + ptrdiff_t(last_ax), iter_strides.end());
+        }
+        else if (e.layout() == layout_type::column_major)
+        {
+            // we got column_major here
+            std::size_t last_ax = merge_loops(axes.begin(), axes.end());
+
+            // erasing the front vs the back
+            iter_shape.erase(iter_shape.begin(), iter_shape.begin() + ptrdiff_t(last_ax + 1));
+            iter_strides.erase(iter_strides.begin(), iter_strides.begin() + ptrdiff_t(last_ax + 1));
+
+            // and reversing, to make it work with the same next_idx function
+            std::reverse(iter_shape.begin(), iter_shape.end());
+            std::reverse(iter_strides.begin(), iter_strides.end());
+        }
+        else
+        {
+            throw std::runtime_error("Layout not supported in immediate reduction.");
+        }
+
+        xindex temp_idx(iter_shape.size());
+        auto next_idx = [&iter_shape, &iter_strides, &temp_idx]()
+        {
+            std::size_t i = iter_shape.size();
+            for (; i > 0; --i)
+            {
+                if (ptrdiff_t(temp_idx[i - 1]) >= ptrdiff_t(iter_shape[i - 1]) - 1)
+                {
+                    temp_idx[i - 1] = 0;
+                }
+                else
+                {
+                    temp_idx[i - 1]++;
+                    break;
+                }
+            }
+            return std::make_pair(i == 0,
+                                  std::inner_product(temp_idx.begin(), temp_idx.end(),
+                                                     iter_strides.begin(), ptrdiff_t(0)));
+        };
+
+        auto begin = e.raw_data();
+        auto out = result.raw_data();
+        auto out_begin = result.raw_data();
+
+        ptrdiff_t next_stride = 0;
+
+        std::pair<bool, ptrdiff_t> idx_res(false, 0);
+
+        // Remark: eventually some modifications here to make conditions faster where merge + accumulate is the
+        // same function (e.g. check std::is_same<decltype(merge_fct), decltype(acc_fct)>::value) ...
+
+        auto merge_border = out;
+        bool merge = false;
+
+        // TODO there could be some performance gain by removing merge checking
+        //      when axes.size() == 1 and even next_idx could be removed for something simpler (next_stride always the same)
+        //      best way to do this would be to create a function that takes (begin, out, outer_loop_size, inner_loop_size, next_idx_lambda)
+        // Decide if going about it row-wise or col-wise
+        if (inner_stride == 1)
+        {
+            while(idx_res.first != true)
+            {
+                // for unknown reasons it's much faster to use a temporary variable and
+                // std::accumulate here -- probably some cache behavior
+                result_type tmp;
+                tmp = init_fct(*begin);
+                tmp = std::accumulate(begin + 1, begin + outer_loop_size, tmp, acc_fct);
+
+                // use merge function if necessary
+                *out = merge ? merge_fct(*out, tmp) : tmp;
+
+                begin += outer_loop_size;
+
+                idx_res = next_idx();
+                next_stride = idx_res.second;
+                out = out_begin + next_stride;
+
+                if (out > merge_border)
+                {
+                    // looped over once
+                    merge = false;
+                    merge_border = out;
+                }
+                else
+                {
+                    merge = true;
+                }
+            };
+        }
+        else
+        {
+            while(idx_res.first != true)
+            {
+                std::transform(out, out + inner_loop_size, begin, out,
+                               [merge, &init_fct, &merge_fct](auto&& v1, auto&& v2)
+                               {
+                                    return merge ? merge_fct(v1, v2) : init_fct(v2);
+                               }
+                );
+
+                begin += inner_stride;
+                for (std::size_t i = 1; i < outer_loop_size; ++i)
+                {
+                    std::transform(out, out + inner_loop_size, begin, out, acc_fct);
+                    begin += inner_stride;
+                }
+
+                idx_res = next_idx();
+                next_stride = idx_res.second;
+                out = out_begin + next_stride;
+
+                if (out > merge_border)
+                {
+                    // looped over once
+                    merge = false;
+                    merge_border = out;
+                }
+                else
+                {
+                    merge = true;
+                }
+
+            };
+        }
+        return result;
+    }
+
     /*************
      * xreducer  *
      *************/
@@ -162,7 +378,7 @@ namespace xt
 
         using substepper_type = typename xexpression_type::const_stepper;
         using value_type = std::decay_t<decltype(std::declval<reduce_functor_type>()(
-            std::declval<init_functor_type>()(**(substepper_type*)0), **(substepper_type*)0))>;
+            std::declval<init_functor_type>()(*std::declval<substepper_type>()), *std::declval<substepper_type>()))>;
         using reference = value_type;
         using const_reference = value_type;
         using pointer = value_type*;
@@ -230,6 +446,22 @@ namespace xt
      * reduce implementation *
      *************************/
 
+    namespace detail
+    {
+        template <class F, class E, class X>
+        inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::lazy) noexcept
+        {
+            using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<X>>;
+            return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+        }
+
+        template <class F, class E, class X>
+        inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::immediate) noexcept
+        {
+            return reduce_immediate(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+        }
+    }
+
     /**
      * @brief Returns an \ref xexpression applying the speficied reducing
      * function to an expresssion over the given axes.
@@ -237,25 +469,23 @@ namespace xt
      * @param f the reducing function to apply.
      * @param e the \ref xexpression to reduce.
      * @param axes the list of axes.
+     * @param evaluation_strategy evaluation strategy to use (lazy (default), or immediate)
      *
      * The returned expression either hold a const reference to \p e or a copy
      * depending on whether \p e is an lvalue or an rvalue.
      */
 
-    template <class F, class E, class X>
-    inline auto reduce(F&& f, E&& e, X&& axes) noexcept
+    template <class F, class E, class X, class ES, class>
+    inline auto reduce(F&& f, E&& e, X&& axes, ES evaluation_strategy) noexcept
     {
-        using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<X>>;
-        return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes), evaluation_strategy);
     }
 
-    template <class F, class E>
-    inline auto reduce(F&& f, E&& e) noexcept
+    template <class F, class E, class ES, class>
+    inline auto reduce(F&& f, E&& e, ES evaluation_strategy) noexcept
     {
         auto ar = arange(e.dimension());
-        using AR = decltype(ar);
-        using reducer_type = xreducer<F, const_xclosure_t<E>, AR>;
-        return reducer_type(std::forward<F>(f), std::forward<E>(e), std::move(ar));
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ar), evaluation_strategy);
     }
 
 #ifdef X_OLD_CLANG
@@ -267,12 +497,12 @@ namespace xt
         return reducer_type(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes));
     }
 #else
-    template <class F, class E, class I, std::size_t N>
-    inline auto reduce(F&& f, E&& e, const I (&axes)[N]) noexcept
+    template <class F, class E, class I, std::size_t N, class ES>
+    inline auto reduce(F&& f, E&& e, const I (&axes)[N], ES evaluation_strategy) noexcept
     {
         using axes_type = std::array<typename std::decay_t<E>::size_type, N>;
-        using reducer_type = xreducer<F, const_xclosure_t<E>, axes_type>;
-        return reducer_type(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes));
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
+
     }
 #endif
 
@@ -298,7 +528,8 @@ namespace xt
         using substepper_type = typename xexpression_type::const_stepper;
         using shape_type = typename xreducer_type::shape_type;
 
-        xreducer_stepper(const xreducer_type& red, size_type offset, bool end = false, layout_type l = layout_type::row_major);
+        xreducer_stepper(const xreducer_type& red, size_type offset, bool end = false,
+                         layout_type l = default_assignable_layout(xexpression_type::static_layout));
 
         reference operator*() const;
 
@@ -535,7 +766,7 @@ namespace xt
         size_type dim = 0;
         while (first != last)
         {
-            stepper.step(dim++, *first++);
+            stepper.step(dim++, std::size_t(*first++));
         }
         return *stepper;
     }
