@@ -6,8 +6,8 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XSTRIDES_HPP
-#define XSTRIDES_HPP
+#ifndef XTENSOR_STRIDES_HPP
+#define XTENSOR_STRIDES_HPP
 
 #include <cstddef>
 #include <functional>
@@ -53,6 +53,16 @@ namespace xt
     void adapt_strides(const shape_type& shape, strides_type& strides,
                        backstrides_type& backstrides) noexcept;
 
+    /*****************
+     * unravel_index *
+     *****************/
+
+    template <class S>
+    S unravel_from_strides(typename S::value_type index, const S& strides, layout_type l);
+
+    template <class S>
+    S unravel_index(typename S::value_type index, const S& shape, layout_type l);
+
     /***********************
      * broadcast functions *
      ***********************/
@@ -63,6 +73,27 @@ namespace xt
     template <class S1, class S2>
     bool broadcastable(const S1& s1, S2& s2);
 
+    /********************************************
+     * utility functions for strided containers *
+     ********************************************/
+
+    template <class C, class It>
+    It strided_data_end(const C& c, It end, layout_type l)
+    {
+        using strides_type = std::decay_t<decltype(c.strides())>;
+        using difference_type = typename std::iterator_traits<It>::difference_type;
+        if (c.dimension() == 0)
+        {
+            return end;
+        }
+        else
+        {
+            auto leading_stride = (l == layout_type::row_major ? c.strides().back() : c.strides().front());
+            leading_stride = std::max(leading_stride, typename strides_type::value_type(1));
+            return end + difference_type(leading_stride - 1);
+        }
+    }
+
     /******************
      * Implementation *
      ******************/
@@ -120,7 +151,8 @@ namespace xt
     template <class size_type, class S, class It>
     inline size_type element_offset(const S& strides, It first, It last) noexcept
     {
-        auto size = std::min(static_cast<typename S::size_type>(std::distance(first, last)), strides.size());
+        using difference_type = typename std::iterator_traits<It>::difference_type;
+        auto size = static_cast<difference_type>(std::min(static_cast<typename S::size_type>(std::distance(first, last)), strides.size()));
         return std::inner_product(last - size, last, strides.cend() - size, size_type(0));
     }
 
@@ -131,7 +163,9 @@ namespace xt
                                   bs_ptr backstrides, typename strides_type::size_type i) noexcept
         {
             if (shape[i] == 1)
+            {
                 strides[i] = 0;
+            }
             (*backstrides)[i] = strides[i] * (shape[i] - 1);
         }
 
@@ -140,7 +174,9 @@ namespace xt
                                   std::nullptr_t, typename strides_type::size_type i) noexcept
         {
             if (shape[i] == 1)
+            {
                 strides[i] = 0;
+            }
         }
 
         template <class shape_type, class strides_type, class bs_ptr>
@@ -203,6 +239,42 @@ namespace xt
         }
     }
 
+    template <class S>
+    inline S unravel_from_strides(typename S::value_type index, const S& strides, layout_type l)
+    {
+        using size_type = typename S::size_type;
+        using value_type = typename S::value_type;
+        S result = xtl::make_sequence<S>(strides.size(), 0);
+        auto lambda = [&index](const value_type& str)
+        {
+            value_type quot = str != 0 ? index / str : 0;
+            index = str != 0 ? index % str : index;
+            return quot;
+        };
+
+        if (l == layout_type::row_major)
+        {
+            std::transform(strides.cbegin(), strides.cend(), result.begin(), lambda);
+        }
+        else if(l == layout_type::column_major)
+        {
+            std::transform(strides.crbegin(), strides.crend(), result.rbegin(), lambda);
+        }
+        else
+        {
+            throw std::runtime_error("unravel_index: dynamic layout not supported yet");
+        }
+        return result;
+    }
+
+    template <class S>
+    inline S unravel_index(typename S::value_type index, const S& shape, layout_type l)
+    {
+        S strides = xtl::make_sequence<S>(shape.size(), 0);
+        compute_strides(shape, l, strides);
+        return unravel_from_strides(index, strides, l);
+    }
+
     template <class S1, class S2>
     inline bool broadcast_shape(const S1& input, S2& output)
     {
