@@ -20,6 +20,7 @@
 #include <utility>
 #include <vector>
 
+#include "xtl/xfunctional.hpp"
 #include "xtl/xtype_traits.hpp"
 
 #include "xtensor_config.hpp"
@@ -77,6 +78,9 @@ namespace xt
     template <class... T>
     using void_t = typename make_void<T...>::type;
 
+    template <class T, class R>
+    using disable_integral_t = std::enable_if_t<!std::is_integral<T>::value, R>;
+
     /*******************************
      * remove_class implementation *
      *******************************/
@@ -596,60 +600,6 @@ namespace xt
         using type = void;
     };
 
-    /*****************************
-     * is_complete implemenation *
-     *****************************/
-
-    namespace detail
-    {
-        template <class T>
-        class is_complete_impl
-        {
-            template <class U>
-            static auto test(U*)  -> std::integral_constant<bool, sizeof(U) == sizeof(U)>;
-
-            static auto test(...) -> std::false_type;
-
-        public:
-
-            using type = decltype(test((T*)0));
-        };
-    }
-
-    template <class T>
-    struct is_complete : detail::is_complete_impl<T>::type {};
-
-    /*************
-     * static_if *
-     *************/
-
-    struct identity_functor
-    {
-        template <class T>
-        T&& operator()(T&& x) const
-        {
-            return std::forward<T>(x);
-        }
-    };
-
-    template <class TF, class FF>
-    auto static_if(std::true_type, const TF& tf, const FF&)
-    {
-        return tf(identity_functor());
-    }
-
-    template <class TF, class FF>
-    auto static_if(std::false_type, const TF&, const FF& ff)
-    {
-        return ff(identity_functor());
-    }
-
-    template <bool cond, class TF, class FF>
-    auto static_if(const TF& tf, const FF& ff)
-    {
-        return static_if(std::integral_constant<bool, cond>(), tf, ff);
-    }
-
     /********************************************
      * xtrivial_default_construct implemenation *
      ********************************************/
@@ -675,7 +625,7 @@ namespace xt
 
     template <class T>
     struct conditional_cast_functor<false, T>
-    : public identity_functor
+    : public xtl::identity
     {
     };
 
@@ -683,7 +633,7 @@ namespace xt
     struct conditional_cast_functor<true, T>
     {
         template <class U>
-        inline auto operator()(U && u) const
+        inline auto operator()(U&& u) const
         {
             return static_cast<T>(std::forward<U>(u));
         }
@@ -698,7 +648,7 @@ namespace xt
      * explicit cast is ok.
      */
     template <bool condition, class T, class U>
-    inline auto conditional_cast(U && u)
+    inline auto conditional_cast(U&& u)
     {
         return conditional_cast_functor<condition, T>()(std::forward<U>(u));
     };
@@ -724,13 +674,13 @@ namespace xt
     template <class T0, class T1>
     struct promote_type<T0, T1>
     {
-        using type = decltype(*(std::decay_t<T0>*)0 + *(std::decay_t<T1>*)0);
+        using type = decltype(std::declval<std::decay_t<T0>>() + std::declval<std::decay_t<T1>>());
     };
 
     template <class T0, class... REST>
     struct promote_type<T0, REST...>
     {
-        using type = decltype(*(std::decay_t<T0>*)0 + *(typename promote_type<REST...>::type*)0);
+        using type = decltype(std::declval<std::decay_t<T0>>() + std::declval<typename promote_type<REST...>::type>());
     };
 
     template <>
@@ -802,7 +752,7 @@ namespace xt
         using std::sqrt;
 
         template <class T>
-        using real_promote_type_t = decltype(sqrt(*(std::decay_t<T>*)0));
+        using real_promote_type_t = decltype(sqrt(std::declval<std::decay_t<T>>()));
     }
 
     /**
@@ -923,7 +873,7 @@ namespace xt
             template <class U>
             static typename U::value_type test(U*, typename U::value_type* = 0);
 
-            using T = decltype(test((ARRAY*)0));
+            using T = decltype(test(std::declval<ARRAY*>()));
 
             static const bool value = !std::is_same<T, void*>::value;
 
